{% extends "rest_framework/login_base.html" %}

{% block branding %}
    <h3 style="margin: 0 0 20px;">Datapunt</h3>
    <p>
    Heeft u geen gebruikersnaam? Bel Arris Oliemans op 0900-NIETDUUR
    </p>
{% endblock %}

{% block bootstrap_theme %}
    <link rel="stylesheet" href="//bootswatch.com/simplex/bootstrap.min.css" type="text/css">
{% endblock %}

